' -----[ Title ]-----------------------------------------------------------
'
' File...... Proj10PB.BAS
' Purpose... PIC16F876 -> 24LC00 Microchip EEPROM
' Author.... Chuck Hellebuyck
' Started... February 9, 2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' This program demonstrates the use of the I2CIN and I2COUT commands.
'
' The program is written to work with a PC. The PC will send serial data to the
'circuit to be stored into the 16 bit EEPROM chip. If a value of 255 is sent to the 
'circuit, then the program will read each EEPROM location and send the data
'to the PC to be displayed. 
'The receiving part of the program requires the address and the value of the
'data to be stored. It then overwrites that address with the received data using
'the I2COUT command.
'When the data is 255, the program uses the I2CIN to read the EEPROM one byte
'at a time and send that data to the PC.
'rite address to the first 16 locations of an external serial EEPROM
' Hardware Connections:
'
' PIC		EEPROM Pin	EEPROM Pin Name	Misc Conn.
' ------		---------------------	--------------------------------	-------------------
' RA0		EEPROM.5	 SDA 			       
' RA1		EEPROM.6	 SCL
' RB1        						Serial In (RX)
' RB2       						Serial Out (TX)
' 		EEPROM .1	A0			Gnd
'		EEPROM. 2	A1			Gnd
'		EEPROM. 3	A2			Gnd
'		EEPROM. 4	Vss			Gnd
'		EEPROM. 7	WP			Gnd
'		EEPROM. 8	Vcc			5v

' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Defines ]-------------------------------------------------------
'


' -----[ Variables and Constants ]-------------------------------------------------------
'
symbol SO = 2                       ' Define serial output pin
symbol SI = 1                       ' Define serial input pin
symbol control = %01010000               ' Set EEPROM control byte
symbol Address = b3                    ' Byte to store address
symbol Value = b2                    ' Byte to store value to store
symbol X = b1                    ' multi-purpose variable
symbol adcon1 = $9f             ' Define adcon1 register address
' -----[ Initialization ]--------------------------------------------------
'
    poke adcon1, 7    
Init:	  
	For address = 0 To 15       	' Loop 16 times
    I2Cout control,address, (10)  	' Preset each address to ten
    Pause 10            	' Delay 10ms after each write
    Next 

' -----[ Main Code ]-------------------------------------------------------
'

RX:
	serout SO, T2400,("Enter #address#value") 'Display instruction line
    Serin SI,T2400,address, value	' Receive location and data to store
	If address = 255 then TX		' Test for data dump request
	I2Cout control, address, (value)	' Store value received at address received
	pause 10				        ' Delay to allow write to occur
	
	Goto RX			             	' Jump back to receive more data



 TX:
	For address = 0 To 15         			' Loop thru 16 locations
    I2Cin control, address, value		' Read byte at address X
    Serout SO,T2400,(#address, ": ",#value, 10, 13) 	' Send Address and value read to PC
    Next 

    Goto RX					' Loop back to the top to receive data
