' -----[ Title ]-----------------------------------------------------------
'
' File...... Proj10PR.BAS
' Purpose... PIC16F876 -> 24LC00 Microchip EEPROM
' Author.... Chuck Hellebuyck
' Started... February 9, 2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' This program demonstrates the use of the I2CIN and I2COUT commands.
'
' The program is written to work with a PC. The PC will send serial data to the
'circuit to be stored into the 16 bit EEPROM chip. If a value of 255 is sent to the 
'circuit, then the program will read each EEPROM location and send the data
'to the PC to be displayed. 
'The receiving part of the program requires the address and the value of the
'data to be stored. It then overwrites that address with the received data using
'the I2COUT command.
'When the data is 255, the program uses the I2CIN to read the EEPROM one byte
'at a time and send that data to the PC.
'rite address to the first 16 locations of an external serial EEPROM
' Hardware Connections:
'
' PIC		EEPROM Pin	EEPROM Pin Name	Misc Conn.
' ------		---------------------	--------------------------------	-------------------
' RA0		EEPROM.5	 SDA 			       
' RA1		EEPROM.6	 SCL
' RB1        						Serial In (RX)
' RB2       						Serial Out (TX)
' 		EEPROM .1	A0			Gnd
'		EEPROM. 2	A1			Gnd
'		EEPROM. 3	A2			Gnd
'		EEPROM. 4	Vss			Gnd
'		EEPROM. 7	WP			Gnd
'		EEPROM. 8	Vcc			5v

' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Defines ]-------------------------------------------------------
'
        Include "modedefs.bas"          ' Include serial modes
        define loader_used 1		'Used for bootloader only


' -----[ Variables and Constants ]-------------------------------------------------------
'
SO      con     2                       ' Define serial output pin
SI      con     1                       ' Define serial input pin
Control con     %10100000               ' Set EEPROM control byte
DPIN    var     PORTA.0                 ' I2C data pin
CPIN    var     PORTA.1                 ' I2C clock pin
Address var     byte                    ' Byte to store address
Value   var     byte                    ' Byte to store value to store
X       var     byte                    ' multi-purpose variable

' -----[ Initialization ]--------------------------------------------------
'
    adcon1 = 7          ' Set PortA to digital ports
    TRISA = %00000000   ' Set PortA as all outputs
    
Init:	  
	For x = 0 To 15       	' Loop 16 times
    I2Cwrite dpin,cpin,control,x,[20]  	' Preset each address to ten
    Pause 10            	' Delay 10ms after each write
    Next 

' -----[ Main Code ]-------------------------------------------------------
'

RX:
	serout SO, T2400,["Enter #address#value"] 'Display instruction line
    Serin SI,T2400,address, value	' Receive location and data to store
	If address = 255 then TX		' Test for data dump request
	I2Cwrite dpin,cpin,control,address,[value]	' Store value received at address received
	pause 10				        ' Delay to allow write to occur
	
	Goto RX			             	' Jump back to receive more data



 TX:
	For X = 0 To 15         			' Loop thru 16 locations
    I2Cread dpin,cpin,control, X, [value]		' Read byte at address X
    Serout SO,T2400,[#X, ": ",#value, 10, 13] 	' Send Address and value read to PC
    Next 

    Goto RX					' Loop back to the top to receive data
