' -----[ Title ]-----------------------------------------------------------
'
' File...... Proj11PB.BAS
' Purpose... PIC16F876 -> Internal PIC16f876 EEPROM
' Author.... Chuck Hellebuyck
' Started... February 9, 2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' This program demonstrates the use of the Read and Write commands.
' The program is written to work with a PC. The PC will send serial data to the
'circuit to be stored into the PICs internal EEPROM. If a value of 255 is sent to the 
'circuit, then the program will read each EEPROM location and send the data
'to the PC to be displayed. 
'To update the existing EEPROM wih new data, the receiving part of the program
'requires the address and the value of the data to be stored.
'It then overwrites that address with the received data using
'the Write command.
'When the data is 255, the program uses the Read command to read the EEPROM one byte
'at a time and send that data to the PC.
'
' Hardware Connections:
'
' Since everything is internal to the PIC there is no external hardware to 
' hookup. Just the standard Oscillator, MCLR, power and ground connections.

' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Defines ]-------------------------------------------------------
'


' -----[ Variables and Constants ]-------------------------------------------------------
'
symbol SO = 2                       ' Define serial output pin
symbol SI = 1                       ' Define serial input pin
symbol Address = b3                    ' Byte to store address
symbol Value = b2                    ' Byte to store value to store
' -----[ Initialization ]--------------------------------------------------
'
Init:	  
	For address = 0 To 15  	' Loop 16 times
    Write address, 10     	' Preset each address to ten
    Pause 10            	' Delay 10ms after each write
    Next 

' -----[ Main Code ]-------------------------------------------------------
'

RX:
	serout SO, T2400,("Enter #address#value") 'Display instruction line
    Serin SI,T2400,address, value	' Receive location and data to store
	If address = 255 then TX		' Test for data dump request
	Write address, value	' Store value received at address received
	pause 10				        ' Delay to allow write to occur
	
	Goto RX			             	' Jump back to receive more data



 TX:
	For address = 0 To 15         			' Loop thru 16 locations
    Read address, value		' Read byte at address X
    Serout SO,T2400,(#address, ": ",#value, 10, 13) 	' Send Address and value read to PC
    Next 

    Goto RX					' Loop back to the top to receive data
