' -----[ Title ]-------------------------------------------------------
'
' File...... proj13pb.BAS
' Purpose... Drive Robot in Unique Pattern PIC16F876 -> Servos
' Author.... Chuck Hellebuyck
' Started... March 1, 2002
' Updated... 


' -----[ Program Description ]-----------------------------------------
'
' This Program uses the 16F876 to control a Servo robot platform. The
' robot has dual servo motors with wheels mounted to drive the robot.
' This program will drive the robot forward a short distance and then
' turn left. After completing the left turn, the robot will drive 
' straight in reverse for a short distance and then stop. After that the 
' robot will turn right. After completing the right turn, the robot will 
' start the routine over with the forward movement.
'
' RA0       GP2D15 Detector Output
' RA1       Left Detector Element Output
' RA2       Center Detector Element Output
' RA3       Right Detector Element Output
' RC1       Left LED
' RC2       Left Center LED
' RC3       Center LED
' RC5       Right Center LED
' RC6       Right LED
' RB2       Right Wheel Servo
' RB5       GP2D15 Servo
' RB7       Left Wheel Servo

' -----[ Revision History ]--------------------------------------------
'
' 

' -----[ Constants ]---------------------------------------------------
'

' -----[ Variables ]---------------------------------------------------
'
symbol move = b0            'robot movement variable 
symbol x = b1               'general purpose variable

' -----[ Initialization ]----------------------------------------------
'

Init:
DIRS = %1111111111          'Set portB to all outputs

' -----[ Main Code ]---------------------------------------------------
'

main:
' *** Move robot forward and pause
move = 100      'Preset move to 100 for distance
gosub fwd       'Jump to fwd subroutine for forward movement
pause 10        'Delay before next move

' *** Make robot turn left 180 degrees and pause
move = 250      'Preset move to 250 for 180 degrees turn
gosub lfwd      'Jump to lfwd subroutine to turn left
pause 10        'Delay before next move

' *** Move robot in reverse and then pause
move = 100      'Preset move to 100 for distance backwards
gosub revrs     'Jump to revrs subroutine to move backwards
pause 10        'Delay before next move

' *** Make robot turn right 180 degrees and pause
move = 200       'Preset move to 200 for 180 degrees turn
gosub rfwd      'Jump to rfwd subroutine to turn right
pause 10        'Delay before next move

goto main       'Loop back and do it again


' *** Move robot forward subroutine
FWD:
For x = 1 to move   'Start forward movement loop
pulsout 2, 100      'Turn right wheel forward
pulsout 7, 200      'Turn left wheel forward
pause 10            'Delay to control robot speed
next                'Repeat forward movement loop
return              'Jump back to where this subroutine was called

' *** Move robot in reverse subroutine
revrs:
For x = 1 to move   'Start reverse movement loop
pulsout 2, 200      'Turn right wheel in reverse
pulsout 7, 100      'Turn left wheel in reverse
pause 10            'Delay to control robot speed
next                'Repeat reverse movement loop
return              'Jump back to where this subroutine was called

' *** Move robot forward-left subroutine
Lfwd:
For x = 1 to move   'Start left-forward movement loop
pulsout 2,100       'Turn right wheel forward to go left
pause 10            'Delay to control robot speed
next                'Repeat left-forward movement loop 
return              'Jump back to where this subroutine was called 

' *** Move robot forward-right subroutine
Rfwd:
For x = 1 to move   'Start right-forward movement loop
pulsout 7, 200      'Turn left wheel forward to go right
pause 10            'Delay to control robot speed
next                'Repeat right-forward movement loop
return              'Jump back to where this subroutine was called


