' -----[ Title ]-------------------------------------------------------
'
' File...... proj14pb.BAS
' Purpose... Robot to follow Black Line PIC16F876 -> IRD Line Detector
' Author.... Chuck Hellebuyck
' Started... March 1, 2002
' Updated... 


' -----[ Program Description ]-----------------------------------------
'
' This Program uses the 16F876 to control a Servo robot platform. The
' robot has an infrared 3 element line detector module mounted under its 
' platform. This program will drive the robot forward while it scans for 
' the black line under the detector. If the black line hits one of the 
' outer elements, the robot will correct back to center as it moves 
' forward. The object is to have the robot follow the line from one end 
' to the other.
'
' RA0       GP2D15 Detector Output
' RA1       Left Detector Element Output
' RA2       Center Detector Element Output
' RA3       Right Detector Element Output
' RC1       Left LED
' RC3       Center LED
' RC6       Right LED
' RB2       Right Wheel Servo
' RB5       GP2D15 Servo
' RB7       Left Wheel Servo

' -----[ Revision History ]--------------------------------------------
'
' 

' -----[ Constants ]---------------------------------------------------
'
symbol LSENS = bit1       'Left detector element
symbol RSENS = bit2       'Right detector element
symbol CSENS = bit3       'Center detector element
symbol adcon1 = $9F       'A/D control register location
symbol trisa = $85        'PortA Tris register location
symbol trisc = $87        'PortC Tris register location
symbol porta = 5          'PortA register location            
symbol portc = 7          'PortC register location
' -----[ Variables ]---------------------------------------------------
'
symbol move = b3            'robot movement variable 
symbol x = b4               'general purpose variable
' -----[ Initialization ]----------------------------------------------
'

Init:
poke ADCON1, 7              'Set portA to digital I/O
poke TRISA, %11111111       'Set portA to all inputs
poke TRISC, %00000000       'Set portC to all outputs
DIRS = %11111111            'Set portB to all outputs

' -----[ Main Code ]---------------------------------------------------
'

main:
peek porta, b0 
if csens = 1 and lsens = 1 and rsens = 1 then error 'Can't find line
if lsens = 0 and rsens = 0 then halt    'Both elements found line
if csens = 0 then center    'Center element on line goto center routine
if lsens = 0 then left      'Left element on line goto left routine
if rsens = 0 then right     'Right element on line goto right routine
goto main                   'Loop back and do it again

' *** Center element on line routine
center:
poke portc, 0   'All LEDs off
poke portc, 3   'Center LED on
move = 5        'Preset move variable to 5
gosub fwd       'Jump to fwd subroutine
goto main       'Jump back to main loop

' *** Left element on line routine
left:
poke portc, 0   'All LEDs off
poke portc, 1   'Left LED on
move = 5        'Preset move variable to 5
gosub lfwd      'Jump to lfwd subroutine
goto main       'Jump back to main loop

' *** Right element on line routine
right:
poke portc, 0   'All LEDs off
poke portc, 6   'Right LED on
move = 5        'Preset move variable to 5
gosub rfwd      'Jump to rfwd subroutine
goto main       'Jump back to main loop

' *** All elements not on line error routine
error:
poke portc, %01001010   'All LEDs on
gosub fwd       'Jump to fwd subroutine
goto main       'Jump back to main loop

' *** Outer elements sense black line
halt:
poke portc, %01000010   'Right,Left LED on, Center LED off
pause 10        'Add delay for 10 msec
goto main       'Jump back to main loop

' *** Move robot forward subroutine
FWD:
For x = 1 to move   'Start forward movement loop
pulsout 2, 100      'Turn right wheel forward
pulsout 7, 200      'Turn left wheel forward
pause 10            'Delay to control robot speed
next                'Repeat forward movement loop
return              'Jump back to where this subroutine was called

' *** Move robot in reverse subroutine
revrs:
For x = 1 to move   'Start reverse movement loop
pulsout 2, 200      'Turn right wheel in reverse
pulsout 7, 100      'Turn left wheel in reverse
pause 10            'Delay to control robot speed
next                'Repeat reverse movement loop
return              'Jump back to where this subroutine was called

' *** Move robot forward-left subroutine
Lfwd:
For x = 1 to move   'Start left-forward movement loop
pulsout 2,100       'Turn right wheel forward to go left
pause 10            'Delay to control robot speed
next                'Repeat left-forward movement loop 
return              'Jump back to where this subroutine was called 

' *** Move robot forward-right subroutine
Rfwd:
For x = 1 to move   'Start right-forward movement loop
pulsout 7, 200      'Turn left wheel forward to go right
pause 10            'Delay to control robot speed
next                'Repeat right-forward movement loop
return              'Jump back to where this subroutine was called


