' -----[ Title ]-----------------------------------------------------------
'
' File...... proj15pb.BAS
' Purpose... Drive Robot with obstacle detection PIC16F876 -> Sharp GP2D15
' Author.... Chuck Hellebuyck
' Started... March 20, 2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' This Program uses the 16F876 to control a Servo robot platform. The robot
' has a servo mounted at its front with a Sharp GP2D15 infrared detector on
' top. This program will drive the robot forward while it scans for 
' obstacles in front of its path. If an obstacle is detected, the robot will
' turn away and then move forward. The object is to have the robot find its 
' way out of a boxed in arena with only one opening for the exit.
'
' RA0       GP2D15 output
' RC1       Left LED (Red)
' RC3       Center LED (green)
' RC6       Right LED (yellow)
' RB2       Right Wheel Servo
' RB5       GP2D15 Servo
' RB7       Left Wheel Servo

' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Constants ]-------------------------------------------------------
'
symbol GP2D15 = bit0      'IRD sensor connected to PortA bit0
symbol adcon1 = $9F       'A/D control register location
symbol trisa = $85        'PortA Tris register location
symbol trisc = $87        'PortC Tris register location
symbol porta = 5          'PortA register location            
symbol portc = 7          'PortC register location

' -----[ Variables ]-------------------------------------------------------
'
symbol position = b2        'obstacle position variable
symbol move = b3            'robot movement variable 
symbol x = b4               'general purpose variable
symbol scan = b5            'GP2D15 position variable

' -----[ Initialization ]--------------------------------------------------
'
Init:
poke ADCON1,7               'Set portA to digital I/O
poke TRISA, %11111111           'Set portA to all inputs
poke TRISC, %00000000           'Set portC to all outputs
DIRS = %11111111           'Set portB to all outputs
scan = 100                  'Preset GP2D15 variable to 100

' -----[ Main Code ]-------------------------------------------------------
'
Main:
poke portc, 0               'Set all LEDs off
position = 0                'Preset position to 0

' *** Scan routine
for scan = 70 to 230 step 10    'Start scanning GP2D15 right to left
pulsout 5, scan                 'Send scanning position signal to servo
pause 20                        'Delay fo Servo to react
peek porta, b0                  'Read porta
if GP2D15 = 0 then skp          'If obstacle not detected, jump to skp
if scan <= 150 then pas        'Test value of scan
position = 1              'Scan less or equal to 150, set position to 1
goto skp                        'Bypass scan > 150 routine
pas:
position = 2              'Scan greater than 150, set position  to 2
skp:
next                            'Loop back for next GP2D15 position

' *** Obstacle Detected

if position <> 0 then decide    'Test for obstacle in previous loop
                                ' if detected jump to decide routine

' *** Obstacle not detected, Forward movement routine
poke portc,0                    'All LEDs off
poke portc, %00001000           'Green LED on
move = 6                        'Preset move variable for forward movement
for x = 1 to move               'Start loop for servo wheel control
pulsout 2, 100                  'Turn Right wheel servo
pulsout 7, 200                  'Turn left wheel servo
pause 10                        'Delay for 10 msec to slow down robot
next                            'Repeat the move robot routine
Ret:
goto main                       'Jump back to main label and start over

decide:
branch position,(main,right,left) 'Choose correction routine
goto main                       'Jump to main routine if position > 2

' *** Left obstacle detected, move right
left:
poke portc, 0           'Turn off all LEDs
poke portc, %00000010   'Turn on Left LED
move = 6                'Preset move variable to 6
for x = 1 to move       'Start turn right routine
pulsout 7, 200          'Turn left wheel servo
pause 30                'Delay to control robot speed
next                    'Repeat the move robot routine
goto ret                'Jump back to main loop at Ret label

' *** Right obstacle detected, move left
right:
poke portc,0            'Turn off all LEDs
poke portc, %01000000   'Turn on Right LED
move = 6                'Preset move variable to 6
for x = 1 to move       'Start turn left routine
pulsout 2, 100          'Turn right wheel servo
pause 30                'Delay to control robot speed
next                    'Repeat the move robot routine
goto ret                'Jump back to main loop at Ret label

goto main               'End program by jumping to main


