' -----[ Title ]-----------------------------------------------------------
'
' File...... proj15pr.BAS
' Purpose... Drive Robot with obstacle detection PIC16F876 -> Sharp GP2D15
' Author.... Chuck Hellebuyck
' Started... March 20, 2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' This Program uses the 16F876 to control a Servo robot platform. The robot
' has a servo mounted at its front with a Sharp GP2D15 infrared detector on
' top. This program will drive the robot forward while it scans for 
' obstacles in front of its path. If an obstacle is detected, the robot will
' turn away and then move forward. The object is to have the robot find its 
' way out of a boxed in arena with only one opening for the exit.
'
' RA0       GP2D15 output
' RC1       Left LED
' RC2       Left Center LED
' RC3       Center LED
' RC5       Right Center LED
' RC6       Right LED
' RB2       Right Wheel Servo
' RB5       GP2D15 Servo
' RB7       Left Wheel Servo

' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Constants ]-------------------------------------------------------
'
LLED var portc.1        'Left LED pin
CLED var portc.3        'Center LED pin
RLED var portc.6        'Right LED pin

' -----[ Variables ]-------------------------------------------------------
'
position var byte           'obstacle position variable
move var byte               'robot movement variable 
x var byte                  'general purpose variable
scan var byte               'GP2D15 position variable

' -----[ Initialization ]--------------------------------------------------
'
       define loader_used 1		'Used for bootloader only

Init:
ADCON1 = 7                  'Set portA to digital I/O
TRISA = %11111111           'Set portA to all inputs
TRISC = %00000000           'Set portC to all outputs
TRISB = %00000000           'Set portB to all outputs
scan = 100                  'Preset GP2D15 variable to 100

' -----[ Main Code ]-------------------------------------------------------
'
Main:
portc = 0                   'Set all LEDs off
position = 0                'Preset position to 0

' *** Scan routine
for scan = 70 to 230 step 10    'Start scanning GP2D15 right to left
pulsout 5, scan                 'Send scanning position signal to servo
pause 20                        'Delay fo Servo to react
if porta.0 = 0 then skp         'If obstacle not detected, jump to skp
if scan <= 150 then pass        'Test value of scan
position = 1              'Scan less or equal to 150, set position to 1
goto skp                        'Bypass scan > 150 routine
pass:
position = 2              'Scan greater than 150, set position  to 2
skp:
next                            'Loop back for next GP2D15 position

' *** Obstacle Detected

if position <> 0 then decide    'Test for obstacle in previous loop
                                ' if detected jump to decide routine

' *** Obstacle not detected, Forward movement routine
portc = 0                       'All LEDs off
high cled                       'Green LED on
move = 6                        'Preset move variable for forward movement
for x = 1 to move               'Start loop for servo wheel control
pulsout 2, 100                  'Turn Right wheel servo
pulsout 7, 200                  'Turn left wheel servo
pause 10                        'Delay for 10 msec to slow down robot
next                            'Repeat the move robot routine
Ret:
goto main                       'Jump back to main label and start over

decide:
branch position,[main,right,left] 'Choose correction routine
goto main                       'Jump to main routine if position > 2

' *** Left obstacle detected, move right
left:
portc = 0               'Turn off all LEDs
high lled               'Turn on Left LED
move = 6                'Preset move variable to 6
for x = 1 to move       'Start turn right routine
pulsout 7, 200          'Turn left wheel servo
pause 30                'Delay to control robot speed
next                    'Repeat the move robot routine
goto ret                'Jump back to main loop at Ret label

' *** Right obstacle detected, move left
right:
portc = 0               'Turn off all LEDs
high rled               'Turn on Right LED
move = 6                'Preset move variable to 6
for x = 1 to move       'Start turn left routine
pulsout 2, 100          'Turn right wheel servo
pause 30                'Delay to control robot speed
next                    'Repeat the move robot routine
goto ret                'Jump back to main loop at Ret label

goto main               'End program by jumping to main


