' -----[ Title ]-----------------------------------------------------------
'
' File...... proj5pbc.BAS
' Purpose... POT -> 16F876 -> LEDs
' Author.... Chuck Hellebuyck
' Started... May 19, 2001
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' This Program uses the 16F876 to read a potentiometer (POT) and drive LEDs
' in a bar graph mode as the POT is turned.
'
' RA2       pot connection          
' RB4       LED4          
' RB3       LED3          
' RB2       LED2          
' RB1       LED1
' RB0       LED0


' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Constants ]-------------------------------------------------------
'

' A/D Variables and symbols
'
Symbol  ADCON0 = $1F                      ' A/D Configuration Register 0
Symbol  ADRESH = $1E                      ' A/D Result for 8-bit mode
Symbol ADRESL = $9E
Symbol  ADCON1 = $9F                    	' A/D Configuration Register 1
Symbol  TRISA  = $85                	' PortA Direction register

' -----[ Variables ]-------------------------------------------------------
'
' B0 and B3 are used but predefined in PBC therefore no symbol required

' -----[ Initialization ]--------------------------------------------------
'
Init:
	pins = $0000                    	' all outputs off to start
	Dirs = %11111111                	' All of Port B is outputs


' -----[ Main Code ]-------------------------------------------------------
'

'************ A/D Read *************************
'
' PEEK and POKE Commands
'
' Access 16F876 A/D using Peek and Poke

Start:
	poke TRISA, $FF         		'Set PortA to Inputs
    	poke ADCON1, $02    		'Set PortA 0-5 to analog inputs, and also
                            			' Sets result to left justified 8-bit mode
    	poke ADCON0, %11010001  	' Set A/D to RC Osc, Channel 2, A/D converter On

loop:
	Peek ADCON0,B0
        	Bit2 = 1
        	Poke ADCON0,B0                  	' Set ADCON0-Bit2 high to start conversion

test: 
	Pause 5
        	Peek ADCON0,B0
        	If Bit2 = 1 Then test		' Wait for low on bit-2 of ADCON0, conversion finished

    	Peek ADRESH,B3                  	' Move HIGH byte of result B3 variable




'********** Drive LEDs *************************

LEDtst1:
	if B3 > 25 then tst2		'If A/D value is less than 25
	pins = %00000001 		' then light LED0 only
	goto cont			'continue with the program

tst2:	
	if B3 > 75 then tst3		'If A/D value is between 25 and 75
	pins = %00000011		' then light LED0 & LED1
	goto cont			'continue with the program

tst3:
	if B3 > 125 then tst4		'If A/D value is between 75 and 125
	pins = %00000111		' then light LED0 - LED2
	goto cont			'continue with the program

tst4:
	if B3 > 175 then tst5		'If A/D value is between 125 and 175
	pins = %00001111 		' then light LED0 - LED3
	goto cont			'continue with the program

tst5:
	pins = %00011111		'A/D value is greater than 175 so 
					' light all the LEDs 0-4

cont:
	Pause 100			'wait 1 second
	goto loop


	end
 
 

