' -----[ Title ]-----------------------------------------------------------
'
' File...... Proj7.BAS
' Purpose... PIC -> LCD (4-bit interface) using 16F876 and 2x16 LCD
' Author.... Chuck Hellebuyck
' Started... November 19, 1999
' Updated... January 20, 2002


' -----[ Program Description ]---------------------------------------------
'
' 
' PIC16F876 to LCD Port predefined connections:
'
' PIC 		    LCD	    	Other Connections
' ------		-------		-----------------------------
' B4       	    LCD.11          
' B5        	LCD.12          
' B6        	LCD.13          
' B7        	LCD.14          
' B3        	LCD.4
' B0        	LCD.6
' OSC1				Resonator - 4 mhz
' OSC2				Resonator - 4 Mhz
' MCLR				Vdd via 1k resistor
' Vdd				5v
' Vss				Gnd


' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Constants ]-------------------------------------------------------
'
' LCD control pins
'
symbol E =  0    ' LCD enable pin (1 = enabled)
symbol RS = 3    ' Register Select (1 = char)

' LCD control characters
'
symbol ClrLCD  =  $01  	    ' clear the LCD
symbol CrsrHm  =  $02       ' move cursor to home position
symbol Row2    =  $C0	    ' 2nd row position of LCD
symbol CrsrLf  =  $10       ' move cursor left
symbol CrsrRt  =  $14       ' move cursor right
symbol DispLf  =  $18       ' shift displayed chars left
symbol DispRt  =  $1C       ' shift displayed chars right
symbol Digit   =  $30       ' Character column code for LCD
' -----[ Variables ]-------------------------------------------------------
'
symbol x = B0            ' General purpose variable
symbol char = B1         ' char sent to LCD
symbol loop1= B2         ' loop counter


' -----[ Initialization ]--------------------------------------------------
'
Init:	pins = $0000     ' all outputs off to start
	Dirs = %11111111     ' LCD pins
	PAUSE 215            ' pause for LCD setup

' Initialize the LCD (Hitatchi HD44780 controller)
'
I_LCD:
	pins = %00110000	'set to 8 bit operation
	PULSOUT E,100		'SEND DATA 3 TIMES
	PAUSE 10
	PULSOUT E,100		'SEND DATA 3 TIMES
	PAUSE 10
	PULSOUT E,100		'SEND DATA 3 TIMES
	PAUSE 10
	PINS = %00100000	'SET TO 4 BIT OPERATION
	pause 1
	PULSOUT E,100		'SEND DATA 3 TIMES
	HIGH RS
	CHAR = %00101000	'4 BIT, 2 LINES, 5X7 CHARACTER
	GOSUB LCDCMD
	CHAR = 8		'Display, cursor and blink off
	GOSUB LCDCMD
	CHAR = 6		'Shift display right
	GOSUB LCDCMD
	CHAR = 1		'clear display and return home
	GOSUB LCDCMD
	CHAR = 15		'display, cursor and blink on
	GOSUB LCDCMD
	

' -----[ Main Loop ]-------------------------------------------------------
'
'Display initial screen
'
'Start:  gosub disp

Start:	char = clrlcd		' Clear LCD	
	gosub lcdcmd			' and position cursor at home
	char = CRSRHM		    ' Issue cursor home command
	gosub lcdcmd			' send cursor home on LCD

'***** Send "Hello World" to first line of LCD ******
	char = "H"			    ' Send "Hello World" one letter
	gosub wrlcd			    '  at a time to the LCD
	char = "e"
	gosub wrlcd
	char = "l"			
	gosub wrlcd		
	char = "l"
	gosub wrlcd
	char = "o"		
	gosub wrlcd		
	char = " "
	gosub wrlcd		
	char = "W"
	gosub wrlcd
	char = "o"		
	gosub wrlcd		
	char = "r"
	gosub wrlcd
	char = "l"
	gosub wrlcd
	char = "d"
	gosub wrlcd

	Pause 1000         'Pause long enough to see it

	goto start

' Send command byte to LCD Subroutine
'
LCDcmd:
    LOW RS                 ' RS low = command
	GOSUB WrLCD            ' send the byte
	HIGH RS                ' return to character mode
	RETURN

' Write ASCII char to LCD Subroutine
'
WrLCD:
    pins = pins & %00001000 ' output high nibble
	b3 = char & %11110000	'store high nibble of char in B3
	pins = pins|b3			'combine RS signal with char
	pause 1				    'wait for data setup
	PULSOUT E, 100          ' strobe the enable line
	b3 = char * 16			'Shift low nibble to high nibble
	pins = pins & %00001000	'combine RS signal with char
	pins = pins|b3	        ' output low nibble     
	pause 1				    'wait for data setup
	PULSOUT E, 100			'strobe the enable line
	RETURN



