' -----[ Title ]-----------------------------------------------------------
'
' File...... proj9pbc.BAS
' Purpose... Serial -> PIC16F876 -> LCD (4-bit interface)
' Author.... Chuck Hellebuyck
' Started... January 20,2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' 
' PIC16F876 Port Hardware connections:
'
' PIC 		    LCD	    	Other Connections
' ------		-------		-----------------------------
' B4       	LCD.11          
' B5        	LCD.12          
' B6        	LCD.13          
' B7        	LCD.14          
' B3        	LCD.4
' B0        	LCD.6
' OSC1				Resonator - 4 mhz
' OSC2				Resonator - 4 Mhz
' MCLR			Vdd via 1k resistor
' Vdd				5v
' Vss				Gnd
' B1                       		Max232(RX)
' B2                       		Max232(TX)

' -----[ Revision History ]------------------------------------------------
'
' 

' -----[ Constants ]-------------------------------------------------------
'
' LCD control pins
'
symbol E =  0                       ' LCD enable pin (1 = enabled)
symbol RS = 3                      ' Register Select (1 = char, 0 = command)

' LCD control characters
'
symbol ClrLCD  =  $01         ' clear the LCD
symbol CrsrHm  =  $02         ' move cursor to home position
symbol Row2    =  $C0 		' 2nd row position of LCD
symbol Row3	= $94			' 3rd row position of LCD
symbol Row4	= $D4			' 4th row position of LCD
symbol CrsrLf  =  $10                     	' move cursor left
symbol CrsrRt  =  $14                     	' move cursor right
symbol DispLf  =  $18                     	' shift displayed chars left
symbol DispRt  =  $1C                     	' shift displayed chars right
symbol Digit   =  $30                     	' Character column code for LCD
' -----[ Variables ]-------------------------------------------------------
'
'B3 reserved for drive routine
symbol x = B0            		' General purpose variable
symbol char = B1         	' char sent to LCD
symbol loop1= B2         	' loop counter
symbol ROW = b5		' LCD ROW value
symbol LOCATION = b6	' Column position on the LCD
symbol VALUE = b7		' Value is the ASCII Character to display
symbol temp2 = b8		' unused
symbol temp3 = b9		' unused
symbol temp4 = b10		' unused
symbol temp1 = b11		' unused

' -----[ Initialization ]--------------------------------------------------
'
Init:	
    	pins = $0000                    	' all outputs off to start
	Dirs = %11111101          	' LCD pins
	PAUSE 215                        	' pause for LCD setup

' Initialize the LCD (Hitatchi HD44780 controller)
'
I_LCD:
	pins = %00110000		'set to 8 bit operation
	PULSOUT E,100		'SEND DATA 3 TIMES
	PAUSE 10
	PULSOUT E,100		'SEND DATA 3 TIMES
	PAUSE 10
	PULSOUT E,100		'SEND DATA 3 TIMES
	PAUSE 10
	PINS = %00100000		'SET TO 4 BIT OPERATION
	pause 1
	PULSOUT E,100		'SEND DATA 3 TIMES
	HIGH RS
	CHAR = %00101000		'4 BIT, 2 LINES, 5X7 CHARACTER
	GOSUB LCDCMD
	CHAR = 8			'Display, cursor and blink off
	GOSUB LCDCMD
	CHAR = 6			'Shift display right
	GOSUB LCDCMD
	CHAR = 1			'clear display and return home
	GOSUB LCDCMD
	CHAR = 15			'display, cursor and blink on
	GOSUB LCDCMD
	

' -----[ Main Code ]-------------------------------------------------------
'
'Display initial screen
'
'******* Main Program ********************************************

Start:	
	SERIN 1,T2400,ROW,LOCATION,VALUE	receive serial data
    	Row = Row - $30			 Correct row to decimal number
    	Location = Location - $30		 Correct location to decimal number

'******* Decision and Branch Routine *****************************
' if ROW = 0 then its a command so jump to command
' if ROW does not = 0 and LOCATION = 0 then write the value where the cursor is.
' VALUE is the ASCII equivalent of the value sent per the LCD ASCII chart

	IF ROW <> 0 AND LOCATION = 0 THEN DISPLAY	Test for zero value
	BRANCH ROW,(COMMAND,R1,R2)',R3,R4)		Branch to proper row

'******* position cursor where new value will be written *********

R1:	
	LOCATION = LOCATION - 1	Correct location byte for zero value
 Shrunk to fit in one line, lookup table for cursor position LCD code
LOOKUP LOCATION,($80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F,$90,$91,$92,$93),char
	GOSUB LCDcmd			Send position command to LCD
	GOTO DISPLAY			 Go to character display routine

R2:
	LOCATION = LOCATION - 1	Correct location byte for zero value
Shrunk to fit in one line, lookup table for cursor position LCD code
LOOKUP LOCATION,($C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF,$D0,$D1,$D2,$D3),char
	GOSUB LCDcmd			Send position command to LCD
	GOTO DISPLAY			 Go to character display routine

**** row 3 and row 4 setup for 4x16 LCD. These command commented out.
'R3:
'	LOCATION = LOCATION - 1
'LOOKUP LOCATION,($94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F,$A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7),char
'	GOSUB LCDcmd
'	GOTO DISPLAY
'
'R4:
'	LOCATION = LOCATION - 1
'LOOKUP LOCATION,($D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF,$E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7),char
'	GOSUB LCDcmd
'	GOTO DISPLAY

'****** convert value to be displayed for wrlcd routine ****************

DISPLAY:
	char = VALUE	 Store received Value character in char variable
	GOSUB WrLCD		 Jump to routine that sends char to LCD
	GOTO START		 jump back to beginning of main loop

'***** ROW=0, Therefore run a command per list below *******************

'0	clear LCD and move to position 1 of row 1
'1	shift cursor left
'2	Display is off, Cursor is off, Cursor Blink is off
'3	Display is on, Cursor is off, Cursor Blink is off
'4	Display is on, Cursor is on, Cursor Blink is off
'5	Display is on, Cursor is off, Cursor Blink is on
'6	Display is on, Cursor is on, Cursor Blink is on
'7	shift display right
'8	shift display left
'9	shift cursor right

COMMAND:
    	value = value - $30		Correct value to decimal for command byte only
	*** Convert byte received to LCD command code
	LOOKUP VALUE,($01,$10,$08,$0C,$0E,$0D,$0F,$1C,$18,$14),char
	GOSUB LCDcmd		Jump to routine that sends command to LCD
	GOTO START		Jump to beginning of main loop
 		
' ****** Send command byte to LCD Routine *************
'LCDcmd: 
	LOW RS                   		' RS low = command
	GOSUB WrLCD              	' send the command byte
	HIGH RS                  		' return to character mode
	RETURN			 Return to caller of this subroutine



'******* Write ASCII char to LCD  **********
WrLCD:
    pins = pins & %00001000 	' Set LCD data lines to zero, leave RS bit alone
	b3 = char & %11110000	'store high nibble of char in B3
	pins = pins|b3			'Output high nibble and RS signal
	pause 1		  	       'Wait for data setup
	PULSOUT E, 100         'Strobe the enable line
	b3 = char * 16			'Shift low nibble to high nibble
	pins = pins & %00001000	'Set LCD data lines to zero, leave RS bit alone
	pins = pins|b3	        		'Output low nibble and RS signal     
	pause 1			'Wait for data setup
	PULSOUT E, 100		Strobe the enable line
	RETURN			 Return to where this subroutine was called		

	END



