' -----[ Title ]-----------------------------------------------------------
'
' File...... proj9pro.BAS
' Purpose... Serial -> PIC16F876 -> LCD (4-bit interface)
' Author.... Chuck Hellebuyck
' Started... January 22 2002
' Updated... 


' -----[ Program Description ]---------------------------------------------
'
' 
' PIC16F876 Port Hardware connections:
'
' PIC 		    LCD	    	Other Connections
' ------		-------		-----------------------------
' B4       	    LCD.11          
' B5        	LCD.12          
' B6        	LCD.13          
' B7        	LCD.14          
' B3        	LCD.4
' B0        	LCD.6
' OSC1				       Resonator - 4 mhz
' OSC2				       Resonator - 4 Mhz
' MCLR				       Vdd via 1k resistor
' Vdd				       5v
' Vss				       Gnd
' B1                       Max232(RX)
' B2                       Max232(TX)

' -----[ Revision History ]------------------------------------------------
'
'
'-----[DEFINES]------------------------------------------------------------

include "modedefs.bas"	'include serout defines
Define LOADER_USED 1     'Only required if bootloader used to program PIC

DEFINE LCD_DREG	PORTB	'Define PIC port used for LCD Data lines
DEFINE LCD_DBIT	4		'Define first pin of portb connected to LCD DB4
DEFINE LCD_RSREG PORTB	'Define PIC port used for RS line of LCD
DEFINE LCD_RSBIT 3		'Define Portb pin used for RS connection
DEFINE LCD_EREG	PORTB	'Define PIC prot used for E line of LCD
DEFINE LCD_EBIT	0		'Define PortB pin used for E connection
DEFINE LCD_BITS	4		'Define the 4 bit communication mode to LCD
DEFINE LCD_LINES 2		'Define using a 2 line LCD
DEFINE LCD_COMMANDUS 2000	'Define delay time between sending LCD commands
DEFINE LCD_DATAUS 50	'Define delay time between data sent.

' -----[ Constants ]-------------------------------------------------------
'
' -----[ Variables ]-------------------------------------------------------
'
ROW var byte		' LCD ROW value
LOCATION var byte	' Column position on the LCD
VALUE var byte  	' Value is the ASCII Character to display
char var byte       ' Temporary storage of character code

' -----[ Initialization ]--------------------------------------------------
'
Init:	
    TRISB = $0000                    ' all outputs off to start
	portb = %11111101                ' LCD pins
    LCDOUT "Serial LCD"              ' Display project name on LCD
    pause 1000                       ' Delay 1 second
    
' -----[ Main Code ]-------------------------------------------------------
'
'Display initial screen
'
'******* Main Program ********************************************

Start:	
	SERIN 1,T2400,ROW,LOCATION,VALUE
    Row = Row - $30
    Location = Location - $30

'******* Decision and Branch Routine *****************************
' if ROW = 0 then its a command so jump to command
' if ROW does not = 0 and LOCATION = 0 then write the value where the cursor is.
' VALUE is the ASCII equivalent of the value sent per the LCD ASCII chart

	IF ROW <> 0 AND LOCATION = 0 THEN DISPLAY
	BRANCH ROW,[COMMAND,Row1,Row2]',Row3,Row4]

'******* position cursor where new value will be written *********

Row1:	
	LOCATION = LOCATION - 1
	LOOKUP LOCATION,[$80,$81,$82,$83,$84,$85,$86,$87,$88,$89,$8A,$8B,$8C,$8D,$8E,$8F,$90,$91,$92,$93],char
	GOSUB LCDcmd
	GOTO DISPLAY

Row2:
	LOCATION = LOCATION - 1
	LOOKUP LOCATION,[$C0,$C1,$C2,$C3,$C4,$C5,$C6,$C7,$C8,$C9,$CA,$CB,$CC,$CD,$CE,$CF,$D0,$D1,$D2,$D3],char
	GOSUB LCDcmd
	GOTO DISPLAY
'
'Row3:
'	LOCATION = LOCATION - 1
'	LOOKUP LOCATION,[$94,$95,$96,$97,$98,$99,$9A,$9B,$9C,$9D,$9E,$9F,$A0,$A1,$A2,$A3,$A4,$A5,$A6,$A7],char
'	GOSUB LCDcmd
'	GOTO DISPLAY
'
'Row4:
'	LOCATION = LOCATION - 1
'	LOOKUP LOCATION,[$D4,$D5,$D6,$D7,$D8,$D9,$DA,$DB,$DC,$DD,$DE,$DF,$E0,$E1,$E2,$E3,$E4,$E5,$E6,$E7],char
'	GOSUB LCDcmd
'	GOTO DISPLAY

'****** convert value to be displayed for wrlcd routine ****************

DISPLAY:
	char = VALUE
	GOSUB WrLCD
	GOTO START

'***** ROW=0, Therefore run a command per list below *******************

'0	clear LCD and move to position 1 of row 1
'1	shift cursor left
'2	Display is off, Cursor is off, Cursor Blink is off
'3	Display is on, Cursor is off, Cursor Blink is off
'4	Display is on, Cursor is on, Cursor Blink is off
'5	Display is on, Cursor is off, Cursor Blink is on
'6	Display is on, Cursor is on, Cursor Blink is on
'7	shift display right
'8	shift display left
'9	shift cursor right

COMMAND:
    value = value - $30
	LOOKUP VALUE,[$01,$10,$08,$0C,$0E,$0D,$0F,$1C,$18,$14],char
	GOSUB LCDcmd
	GOTO START
 		
' send command byte to LCD
'
LCDcmd: 
	LCDOUT $FE, char                   ' RS low = command
	RETURN

' Write ASCII char to LCD 
'
WrLCD:
    LCDOUT char
	RETURN

	END


